require("vector")

function love.load()
	Width,Height = love.graphics.getDimensions()
	
	ShowInfo = true;
	
	Fractal = {
		Offset     = Vector.new(.95,1.1,.5),
		Rotation   = Vector.new(-0.01,-0.01,-0.01),
		Iterations = 5
	}

	Camera = {
		Position    = Vector.new(0,1,3),
		Velocity    = Vector.new(0,0,0),
		Forward     = Vector.new(0,0,-1),
		Right       = Vector.new(-1,0,0),
		Upward      = Vector.new(0,1,0),
		Aperture    = 0.0001,
		Focal       = 2.
	}
	
	MovementSpeed     = 1.
	
	RenderCanvas      = love.graphics.newCanvas(nil,nil,rgba32f)--nil uses window size
	Frames            = 0;
	
	RenderShader      = love.graphics.newShader("mengersponge.frag")
	PostShader        = love.graphics.newShader("posteffect.frag")
	
	startTime         = love.timer.getTime()
	
	Render            = false;
	
	Scancodes = {
		["w"]      = love.keyboard.getScancodeFromKey("w"),
		["a"]      = love.keyboard.getScancodeFromKey("a"),
		["s"]      = love.keyboard.getScancodeFromKey("s"),
		["d"]      = love.keyboard.getScancodeFromKey("d"),
		["space"]  = love.keyboard.getScancodeFromKey("space"),
		["lshift"] = love.keyboard.getScancodeFromKey("lshift"),
		["r"]      = love.keyboard.getScancodeFromKey("r"),
		["o"]      = love.keyboard.getScancodeFromKey("o"),
		["["]      = love.keyboard.getScancodeFromKey("["),
		["]"]      = love.keyboard.getScancodeFromKey("]"),
		["."]      = love.keyboard.getScancodeFromKey("."),
		[","]      = love.keyboard.getScancodeFromKey(","),
		["-"]      = love.keyboard.getScancodeFromKey("-"),
		["="]      = love.keyboard.getScancodeFromKey("="),
	}
end


function ResetRender()
	RenderCanvas = love.graphics.newCanvas(nil,nil,rgba32f)
	Frames = 0;
end

function UpdateShader()
	RenderShader:send('iResolution',{Width,Height})
    RenderShader:send('iGlobalTime',love.timer.getTime()-startTime)
	
	RenderShader:send('CameraPosition',{Camera.Position.x,Camera.Position.y,Camera.Position.z})
	RenderShader:send('CameraForward',{Camera.Forward.x,Camera.Forward.y,Camera.Forward.z})
	RenderShader:send('CameraUpward',{Camera.Upward.x,Camera.Upward.y,Camera.Upward.z})
	RenderShader:send('CameraRight',{Camera.Right.x,Camera.Right.y,Camera.Right.z})
	RenderShader:send('CameraAperture',Camera.Aperture)
	RenderShader:send('CameraFocal',Camera.Focal)

	RenderShader:send('FractalOffset',{Fractal.Offset.x,Fractal.Offset.y,Fractal.Offset.z})
	RenderShader:send('FractalRotation',{Fractal.Rotation.x,Fractal.Rotation.y,Fractal.Rotation.z})
	RenderShader:send('FractalIterations', math.floor(Fractal.Iterations))
	
	RenderShader:send('Render',Render)
end

function love.resize(w, h)
  print(("Window resized to width: %d and height: %d."):format(w, h))
  Width = w
  Height = h
  ResetRender()
end

function love.keypressed(key,scancode)
	if key == "b" then
		screenshot = love.graphics.newScreenshot( )
	    local screenshot = love.graphics.newScreenshot();
		screenshot:encode('png', os.time() .. '.png');
	elseif key == "i" then
		ShowInfo = not ShowInfo
	elseif key == "k" then
		Render = not Render
		ResetRender()
	elseif scancode == Scancodes["-"] then
		Fractal.Iterations = math.max(Fractal.Iterations - 1,1)
	elseif scancode == Scancodes["="] then
		Fractal.Iterations = math.min(Fractal.Iterations + 1,10)
	end
end

function love.wheelmoved(x, y)
    if y > 0 then
        MovementSpeed = math.min(MovementSpeed*1.5,2.);
    elseif y < 0 then
        MovementSpeed = math.max(MovementSpeed/1.5,.0001);
    end
end
 

function love.update(dt)
	if(not Render)then
		if love.mouse.isDown(1) then
			local x, y = love.mouse.getPosition()
			x, y = ( x * 2 - Width ) / Height, ( y * 2 - Height ) / Height
			
			Camera.Forward = Vector.rotate(Camera.Forward, x * dt, Camera.Upward )
			Camera.Forward = Vector.rotate(Camera.Forward, y * dt, Camera.Right  )

			Camera.Upward = Vector.normalize(Vector.cross(Camera.Forward, Camera.Right ))
			Camera.Right  = Vector.normalize(Vector.cross(Camera.Upward, Camera.Forward ))
		end
			
		if love.keyboard.isScancodeDown(Scancodes["o"]) then
			if love.keyboard.isScancodeDown(Scancodes["w"]) then
				Fractal.Offset.x = Fractal.Offset.x + .1*dt;
			end
			if love.keyboard.isScancodeDown(Scancodes["s"]) then
				Fractal.Offset.x = Fractal.Offset.x - .1*dt;
			end
			if love.keyboard.isScancodeDown(Scancodes["d"]) then
				Fractal.Offset.z = Fractal.Offset.z + .1*dt;
			end
			if love.keyboard.isScancodeDown(Scancodes["a"]) then
				Fractal.Offset.z = Fractal.Offset.z - .1*dt;
			end
		   if love.keyboard.isScancodeDown(Scancodes["space"]) then
				Fractal.Offset.y = Fractal.Offset.y + .1*dt;
			end
			if love.keyboard.isScancodeDown(Scancodes["lshift"]) then
				Fractal.Offset.y = Fractal.Offset.y - .1*dt;
			end
		elseif love.keyboard.isScancodeDown(Scancodes["r"]) then
			if love.keyboard.isScancodeDown(Scancodes["w"]) then
				Fractal.Rotation.x = Fractal.Rotation.x + .01*dt;
			end
			if love.keyboard.isScancodeDown(Scancodes["s"]) then
				Fractal.Rotation.x = Fractal.Rotation.x - .01*dt;
			end
			if love.keyboard.isScancodeDown(Scancodes["d"]) then
				Fractal.Rotation.z = Fractal.Rotation.z + .01*dt;
			end
			if love.keyboard.isScancodeDown(Scancodes["a"]) then
				Fractal.Rotation.z = Fractal.Rotation.z - .01*dt;
			end
		   if love.keyboard.isScancodeDown(Scancodes["space"]) then
				Fractal.Rotation.y = Fractal.Rotation.y + .01*dt;
			end
			if love.keyboard.isScancodeDown(Scancodes["lshift"]) then
				Fractal.Rotation.y = Fractal.Rotation.y - .01*dt;
			end
		else
			if love.keyboard.isScancodeDown(Scancodes["w"]) then
				Camera.Velocity = Camera.Velocity + Camera.Forward*(dt*MovementSpeed)
			end
			if love.keyboard.isScancodeDown(Scancodes["s"]) then
				Camera.Velocity = Camera.Velocity - Camera.Forward*(dt*MovementSpeed)
			end
			if love.keyboard.isScancodeDown(Scancodes["d"]) then
				Camera.Velocity = Camera.Velocity + Camera.Right*(dt*MovementSpeed)
			end
			if love.keyboard.isScancodeDown(Scancodes["a"]) then
				Camera.Velocity = Camera.Velocity - Camera.Right*(dt*MovementSpeed)
			end
			if love.keyboard.isScancodeDown(Scancodes["space"]) then
				Camera.Velocity = Camera.Velocity + Camera.Upward*(dt*MovementSpeed)
			end
			if love.keyboard.isScancodeDown(Scancodes["lshift"]) then
				Camera.Velocity = Camera.Velocity - Camera.Upward*(dt*MovementSpeed)
			end
		end
		
		if love.keyboard.isScancodeDown(Scancodes["["]) then
			Camera.Aperture = Camera.Aperture*.25^dt;
		end
		if love.keyboard.isScancodeDown(Scancodes["]"]) then
			Camera.Aperture = math.min(Camera.Aperture/.25^dt,1);
		end
		
		if love.keyboard.isScancodeDown(Scancodes[","]) then
			Camera.Focal = Camera.Focal*.5^dt;
		end
		if love.keyboard.isScancodeDown(Scancodes["."]) then
			Camera.Focal = math.min(Camera.Focal/.5^dt,10);
		end
	
		--Limit velocity
		AbsoluteVelocity = Vector.length(Camera.Velocity)
		if(AbsoluteVelocity>0)then
			Camera.Velocity = Camera.Velocity*(math.min(MovementSpeed,AbsoluteVelocity)/AbsoluteVelocity)
		end
	
		--Drag
		Camera.Velocity = Camera.Velocity*(0.5^dt)
	
		--Integrate Camera.Velocity
		Camera.Position = Camera.Position + Camera.Velocity*dt
	end
	
	UpdateShader()
end

function love.draw()
	--Create a new frame
	newRenderFrame = love.graphics.newCanvas(nil,nil,rgba32f)
	love.graphics.setCanvas(newRenderFrame)
		love.graphics.setShader(RenderShader)
			love.graphics.rectangle("fill",0,0,Width,Height)
		love.graphics.setShader()
	love.graphics.setCanvas()
	
	--Add it to the render
	love.graphics.setCanvas(RenderCanvas)
		love.graphics.setColor(255,255,255,255/(Frames+1))
		love.graphics.draw(newRenderFrame)
	love.graphics.setCanvas()
		
	--post process the render
	love.graphics.setShader(PostShader)
		love.graphics.draw(RenderCanvas)
	love.graphics.setShader()
	
	if Render then
		Frames = Frames + 1
	end
	
	--extra info
	if (ShowInfo) then
		love.graphics.setColor(0,0,0)
		love.graphics.rectangle("fill",0,0,400,165)
		love.graphics.setColor(255,255,255)
		love.graphics.print("press 'b' to make a screenshot",0,0)
		love.graphics.print("press 'i' to remove this info",0,15)
		love.graphics.print("press 'k' to switch to ULTIMATE RENDER MODE",0,30)
		
		love.graphics.print("Camera. Position (WASD+Shift+Space) : "..Vector.tostring(Camera.Position,100),0,45)
		love.graphics.print("Camera. Forward (Mouse Drag) : "..Vector.tostring(Camera.Forward,100),0,60)
		love.graphics.print("Camera. Aperture ( [, ] ) : "..string.format("%e",Camera.Aperture),0,75)
		love.graphics.print("Camera. Focal Length ( <, > ) (Shown in Red) : "..string.format("%e",Camera.Focal),0,90)
		
		love.graphics.print("Fractal Offset (Hold 'o' + Controls) : "..Vector.tostring(Fractal.Offset,1000),0,105)
		love.graphics.print("Fractal Rotation (Hold 'r' + Controls) : "..Vector.tostring(Fractal.Rotation,10000),0,120)
		love.graphics.print("Fractal Iterations ( -, + ) : "..Fractal.Iterations,0,135)
		
		love.graphics.print("Movement Speed (Mouse Wheel) : "..MovementSpeed,0,150)
	end
	
end