Vector = {
	new = function(x,y,z)
		v = {x=x,y=y,z=z}
		setmetatable(v,Vector.metatable)
		return v
	end,
	
	length = function(v)
		return math.sqrt(v.x^2+v.y^2+v.z^2)
	end,
	
	normalize = function(v)
		return v/Vector.length(v)
	end,
	
	cross = function(v1,v2)
		return Vector.new(v1.y*v2.z-v1.z*v2.y,
				v1.z*v2.x-v1.x*v2.z,
				v1.x*v2.y-v1.y*v2.x)
	end,
	
	spherical = function(t,p)
		return Vector.new(
			math.cos(t)*math.sin(p),
			math.sin(t),
			math.cos(t)*math.cos(p)
		)
	end,
	
	rotate = function( v1, angle, v2 )--rotates v1 around v2 with angle radians
		local c = math.cos( angle )
		local s = math.sin( angle )

		return Vector.new(
				  v1.x * ( c + v2.x * v2.x * ( 1 - c )            ) +
				  v1.y * (     v2.x * v2.y * ( 1 - c ) - v2.z * s ) +
				  v1.z * (     v2.x * v2.z * ( 1 - c ) + v2.y * s ),

				  v1.x * (     v2.y * v2.x * ( 1 - c ) + v2.z * s ) +
				  v1.y * ( c + v2.y * v2.y * ( 1 - c )            ) +
				  v1.z * (     v2.y * v2.z * ( 1 - c ) - v2.x * s ),

				  v1.x * (     v2.z * v2.x * ( 1 - c ) - v2.y * s ) +
				  v1.y * (     v2.z * v2.y * ( 1 - c ) + v2.x * s ) +
				  v1.z * ( c + v2.z * v2.z * ( 1 - c )            )
		)
	end,
	
	tostring = function(a,d)
		return math.floor(a.x*d)/d..","..math.floor(a.y*d)/d..","..math.floor(a.z*d)/d
	end
}
	
Vector.metatable = {
	__add = function(a,b) return Vector.new(a.x+b.x, a.y+b.y, a.z+b.z) end,
	__sub = function(a,b) return Vector.new(a.x-b.x, a.y-b.y, a.z-b.z) end,
	__mul = function(a,c) return Vector.new(a.x*c, a.y*c, a.z*c) end,
	__div = function(a,c) return Vector.new(a.x/c, a.y/c, a.z/c) end
}




